【问题标题】:CUDA OpenGL interop, resource mapping messes up bufferCUDA OpenGL互操作,资源映射搞乱缓冲区
【发布时间】:2013-04-21 07:05:17
【问题描述】:

我最近开始编写 CUDA OpenGL 互操作性。大多数时候,我一直在 Linux 环境中进行开发。尝试将我的代码传输到运行 OS X 的 MacBook Pro 时遇到以下问题:

每当我将 OpenGL 中的顶点缓冲区对象作为资源映射到 CUDA 时,都会弄乱存储在相应缓冲区中的数据。

这是我的代码:

// OpenGL and window handler includes
#include <GL/glew.h>
#include <SDL/SDL.h>
#include <OpenGL/GL.h>

// OpenGL/CUDA interop
#include <cuda_runtime.h>
#include <cuda_gl_interop.h>

// C libraries
#include <iostream>

int main(int argc, char* argv[])
{
    GLuint vbo;
    struct cudaGraphicsResource *cuda_vbo_resource;

    // create OpenGL context with SDL
    SDL_Init(SDL_INIT_VIDEO);
    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
    SDL_SetVideoMode( 400, 200, 16, SDL_OPENGL);

    // initialize extension wrangler
    glewInit();

    // set cuda/GL device
    cudaGLSetGLDevice(0);

    // generate buffer
    glGenBuffers(1, &vbo);

    // fill buffer with data (1, 2, 3)
    glBindBuffer(GL_ARRAY_BUFFER, vbo);
    int inData[] = {1,2,3};
    glBufferData(GL_ARRAY_BUFFER, 3*sizeof(int), inData, GL_DYNAMIC_DRAW);
    glBindBuffer(GL_ARRAY_BUFFER, 0);

    // Register buffer with cuda
    cudaGraphicsGLRegisterBuffer(&cuda_vbo_resource, vbo,             
                                        cudaGraphicsMapFlagsWriteDiscard);

    // let's have a look at the buffer
    glBindBuffer(GL_ARRAY_BUFFER, vbo);
    int* mappedBuffer = (int *) glMapBuffer(GL_ARRAY_BUFFER,GL_READ_ONLY);
    printf("\tbefore mapping: %d, %d, %d\n",mappedBuffer[0], mappedBuffer[1], 
            mappedBuffer[2]);
    glUnmapBuffer(GL_ARRAY_BUFFER);
    glBindBuffer(GL_ARRAY_BUFFER, 0);

    // map and unmap the cuda resource
    if (cudaGraphicsMapResources(1, &cuda_vbo_resource, 0) != cudaSuccess)
        printf("Resource mapping failed...\n");
    if (cudaGraphicsUnmapResources(1, &cuda_vbo_resource, 0) != cudaSuccess)
        printf("Resource unmapping failed...\n");

    // let's have a look at the buffer again
    glBindBuffer(GL_ARRAY_BUFFER, vbo);
    mappedBuffer = (int *) glMapBuffer(GL_ARRAY_BUFFER,GL_READ_ONLY);
    printf("\tafter mapping: %d, %d, %d\n",mappedBuffer[0], mappedBuffer[1], 
            mappedBuffer[2]);
    glUnmapBuffer(GL_ARRAY_BUFFER);
    glBindBuffer(GL_ARRAY_BUFFER, 0);

    // unregister the cuda resource
    cudaGraphicsUnregisterResource(cuda_vbo_resource);

    // delete the buffer
    glBindBuffer(1, vbo);
    glDeleteBuffers(1, &vbo);

    return 0;
}

编译后:

g++ -o resourceMapping resourceMapping.cpp `sdl-config --cflags --libs` -lglew 
         -framework OpenGL -I /usr/local/cuda/include  -L /usr/local/cuda/lib -lcudart

我明白了

before mapping: 1, 2, 3
after mapping:  0, 33554432, 32

有人知道为什么映射会改变缓冲区数据吗?这让我特别困惑,因为这不会发生在 Linux 机器上。

感谢任何 cmets 或提示!

【问题讨论】:

    标签: opengl cuda


    【解决方案1】:

    我注意到您使用 cudaGraphicsMapFlagsWriteDiscard 注册了 vbo。

    来自here,“指定 CUDA 不会从该资源中读取,而是会覆盖该资源的全部内容,因此不会保留之前存储在该资源中的任何数据。”

    也许试试 cudaGraphicsMapFlagsNone 或 cudaGraphicsMapFlagsReadOnly?

    【讨论】:

      猜你喜欢
      • 2013-04-04
      • 1970-01-01
      • 2014-06-21
      • 2013-01-15
      • 1970-01-01
      • 1970-01-01
      • 2011-09-22
      • 2015-12-13
      • 2011-04-16
      相关资源
      最近更新 更多